COMMENT
Based on src/nrnoc/expsyn.mod
A modification of ExpSyn that scales the amplitude of the 
conductance change by an integer multiple that is governed 
by the binomial distribution.

Note:
Each afferent stream might have its own n and p1, and if it did,
it should also have its own p[] array.
The params n and p1 would then be specified as part of the 
NetCon declaration.
However, for this simple demonstration, n and p1 are RANGE variables, 
and the same n, p1, and p[] apply to all input streams that converge 
on a particular instance of this mechanism.
ENDCOMMENT

NEURON {
	POINT_PROCESS BinExpSyn
	RANGE tau, e, i
	RANGE n, p1, k
	NONSPECIFIC_CURRENT i
}

UNITS {
	(nA) = (nanoamp)
	(mV) = (millivolt)
	(uS) = (microsiemens)
}

DEFINE MAXn 10 : use this to limit maximum n
DEFINE MAXnplus1 11 : NMODL can't handle declaration p[MAXn+1]

PARAMETER {
	tau = 0.1 (ms) <1e-9,1e9>
	e = 0 (mV)
	: if no sites can release, why bother?
	n = 1 (1) <1,MAXn> : number of sites that can release a vesicle
	:         <1, MAXn> won't limit range of n!
	p1 = 0.3 (1) <0,1> : probability that a particular site releases its vesicle
	:        ^^^ must be present or range of p1 won't be limited!
}

ASSIGNED {
	v (mV)
	i (nA)
	k : number of quanta released
	p[MAXnplus1] : p[k] is the probability that k is the total # of quanta released
}

STATE {
	g (uS)
}

PROCEDURE seed(x) {
        set_seed(x)
}

LOCAL j, f

: Thinking ahead to using this in a model that involves 
: use-dependent plasticity of n and/or p1
: the arguments are the current values of n and p1
PROCEDURE pset(nn, pp) {
  : We don't really have to calculate p[nn] because we'll never use it
  if (pp==0) {
    FROM j=0 TO nn {
      p[j]=0
    }
  } else if (pp==1) {
    FROM j=0 TO nn-1 {
      p[j]=0
    }
    p[nn]=1
  } else {
    p[0]=(1-pp)^nn
    p[nn]=pp^nn
    f = pp/(1-pp)
    if (nn>1) {
      FROM j=1 TO nn-1 {
        p[j]=p[j-1]*(nn+1-j)*f/j
      }
    }
  }
}


INITIAL {
	g=0
	pset(n,p1)
: printf("\n")
: printf("%d",n)
: FROM j=0 TO n {
:   printf(" %f",p[j])
: }
: printf("\n")
: f = 0
: FROM j=0 TO n {
:   f = f+p[j]
:   printf(" %f",f)
: }
: printf("\n")
}

BREAKPOINT {
	SOLVE state METHOD cnexp
	i = g*(v - e)
}

DERIVATIVE state {
	g' = -g/tau
}


NET_RECEIVE(weight (uS)) {
:	f = random() : uniform distribution
	: but this gets me "Warning: random undefined. (declared within VERBATIM?)
	f = scop_random() : uniform distribution
: printf("f = %g",f)
: here I would like to use a while loop
: but will resort instead to this kludge
	k = 0
	FROM j=0 TO n-1 {
		f = f-p[j]
		if (f>=0) {
			k = k+1
		}
	}
: printf("  k = %g\n",k)
	state_discontinuity(g, g + weight*k)
}
