COMMENT
Based on src/nrnoc/expsyn.mod
A modification of ExpSyn that scales the amplitude of the 
conductance change by a factor governed by the normal 
distribution.  Negative values are clipped to 0.

The conceptual model is that activation of a synapse with 
nominal weight w produces a postsynaptic conductance 
change with mean value w and variance s*w.  The actual 
conductance change is always >=0 (negative values are 
clipped to 0).

This is based on the notion of a presynaptic terminal 
that releases a single vesicle upon synaptic activation.
The mean transmitter content in each vesicle is x, 
and the variance is s*x.

In this implementation, the user specifies w via the NetCon 
but s is a RANGE variable with a default value of 0.  
Thus, each input stream to a particular instance of this 
mechanism may have its own w, but the same s will apply to 
all input streams that converge on this instance.
An alternative implementation might give each afferent stream 
its own s.  The parameter s would then be specified as part 
of the NetCon declaration.
ENDCOMMENT

NEURON {
	POINT_PROCESS NormExpSyn
	RANGE tau, e, i
	RANGE s, k
	NONSPECIFIC_CURRENT i
}

UNITS {
	(nA) = (nanoamp)
	(mV) = (millivolt)
	(uS) = (microsiemens)
}

PARAMETER {
	tau = 0.1 (ms) <1e-9,1e9>
	e = 0 (mV)
	s = 0 (1) <0,1> : standard deviation
}

ASSIGNED {
	v (mV)
	i (nA)
	k : multiplier of synaptic weight
}

STATE {
	g (uS)
}

PROCEDURE seed(x) {
        set_seed(x)
}

INITIAL {
	g=0
}

BREAKPOINT {
	SOLVE state METHOD cnexp
	i = g*(v - e)
}

DERIVATIVE state {
	g' = -g/tau
}


NET_RECEIVE(weight (uS)) {
	k = normrand(1,s)
	if (k>0) {
: printf("  k = %g\n",k)
		state_discontinuity(g, g + weight*k)
	}
}
