COMMENT
Based on src/nrnoc/expsyn.mod
A modification of ExpSyn that scales the amplitude of the 
conductance change by an integer factor that is governed 
by the Poisson distribution.

poisrand() in NEURON 5.3 2002/10/31 and earlier versions
has a bug in that the integers it returns are 1 larger 
than they should be.  This bug will be fixed in a future 
release of NEURON, but in the meantime it is necessary 
to write code that compensates for it.

This mod file checks for the bug during initialization, 
and if the bug is present, 1 is subtracted from the values 
returned by poisrand().  This is an inefficient hack 
because the bug test is done at each initialization.  
However, the correction factor "bugfix" is GLOBAL 
and is used to control performace of the test, 
so no matter how many instances of this mechanism are 
present in a model, the test will be performed only 
once per initialization.
ENDCOMMENT

NEURON {
	POINT_PROCESS PoissonExpSyn
	RANGE tau, e, i, m, k
	NONSPECIFIC_CURRENT i
	GLOBAL bugfix
}

UNITS {
	(nA) = (nanoamp)
	(mV) = (millivolt)
	(uS) = (microsiemens)
}

DEFINE MAXm 10 : use this to limit maximum m

PARAMETER {
	tau = 0.1 (ms) <1e-9,1e9>
	e = 0 (mV)
	m = 1 (1) <0,MAXm> : mean "quantal" content, a multiplier of synaptic weight
	bugfix = -1 : -1 means test is yet to be done, 0 means no bug, 1 means bug is present
}

ASSIGNED {
	v (mV)
	i (nA)
	k (1) : number of quanta released
}

STATE {
	g (uS)
}

PROCEDURE seed(x) {
        set_seed(x)
}

LOCAL a : used just to check for bug in poisrand()

INITIAL {
	g=0
	: must do this at every init, else first run will produce 
	: a different sequence than subsequent runs do
	a = poisrand(1e-9) : a should be 0
	if (bugfix<0) {
: printf("checking for bug\n")
: printf("a = %g\n", a)
		if (a!=0) { : bug is present
			bugfix = 1
		} else {
			bugfix = 0
		}
: printf("bugfix = %g\n", bugfix)
	}
}

BREAKPOINT {
	SOLVE state METHOD cnexp
	i = g*(v - e)
}

DERIVATIVE state {
	g' = -g/tau
}

NET_RECEIVE(weight (uS)) {
:	state_discontinuity(g, g + weight*poisrand(m))
	k = poisrand(m) - bugfix
: printf("%g\n",k)
	state_discontinuity(g, g + weight*k)
}
